% Landau gauge parallel to the x direction  -  Based on EQuUs v4.8
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @file Landaux.m
%> @brief Vector potential in the Landau gauge parallel to the x direction.
%% Landaux
%> @brief Vector potential in the Landau gauge parallel to the x direction.
%> @param x X coordinates of the sites.
%> @param y Y coordinates of the sites.
%> @param eta_B Dimensionless strength of the magnetic field.
%> @param Aconst A constant vectorpotential .
%> @param height The number of unit cells in the scattering region.
%> @param lattice_constant The lattice constant in the scattering region.
%> @return Returns with N x 2 array containing the x and y components of the vector potential. N is the number of sites.
    function ret = Landaux( x,y, eta_B, Aconst, height, lattice_constant )
    
        %> preallocating array
        ret = zeros( length(x),2 );
            
        %> Identifies region in the first lead (B=0)
        indexes1 = logical( y<=0 );
        ret( indexes1, 1 ) = -Aconst;
           
        %> Identifies region in the second lead (B=0)
        indexes2 = logical( y>=height*lattice_constant );
        ret( indexes2, 1 ) = -eta_B*height*lattice_constant-Aconst;
            
        %> Identifies region in the scattering region (B>0)
        indexes3 = ~( indexes1 | indexes2 );
        ret( indexes3, 1) = -eta_B*y(indexes3)-Aconst;
    end