%   Scalar gauge fiald for transformation from Landauy to Landaux -  Based on EQuUs v4.8
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @file gauge_field.m
%> @brief Scalar gauge field connecting Landaux and Landauy gauges.
%
%% gauge_field
%> @brief Scalar gauge field connecting Landaux and Landauy gauges.
%> @param x X coordinates of the sites.
%> @param y Y coordinates of the sites.
%> @param eta_B Dimensionless strength of the magnetic field.
%> @param Aconst A constant vectorpotential .
%> @param height The number of unit cells in the scattering region.
%> @param lattice_constant The lattice constant in the scattering region.
%> @return Returns with N x 1 array containing the scalar potential field. N is the number of sites.
     function ret = gauge_field(x,y, eta_B, Aconst, height, lattice_constant)
     
        %> preallocating array
        ret = zeros( length(x),1 );
            
        %> Identifies region in the first lead (B=0)
        indexes1 = logical( y<=0 );
        ret( indexes1 ) = -Aconst*x(indexes1);
            
        %> Identifies region in the second lead (B=0)
        indexes2 = logical( y>=height*lattice_constant );
        ret( indexes2 ) = -eta_B*height*lattice_constant*x(indexes2) - Aconst*x(indexes2);
            
        %> Identifies region in the scattering region (B>0)
        indexes3 = ~( indexes1 | indexes2 );
        ret( indexes3 ) = -eta_B*x(indexes3).*y(indexes3) - Aconst*x(indexes3);  
    end