#include "fintrf.h"
C======================================================================
C     Gateway routine to call the zggev.f function from LAPACK package.
C     (See: http://www.netlib.org/lapack/explore-html/d9/d52/dggev_8f.html for details)
C     Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
C
C     This program is free software: you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation, either version 3 of the License, or
C     (at your option) any later version.
C 
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program.  If not, see http://www.gnu.org/licenses/.
C     
C======================================================================
C     Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C     Declarations
      implicit none

C     mexFunction arguments:
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs


C     Function declarations:
      mwPointer mxGetPr
      mwPointer mxGetPi
      mwPointer mxCreateDoubleMatrix
      mwPointer mxGetM, mxGetN
      integer*4 mxIsNumeric
      integer*4 mxIsComplex

      

C     Pointers to input/output mxArrays:
      mwPointer ALPHAR_ptr
      mwPointer ALPHAI_ptr
      mwPointer BETA_ptr
      mwPointer lvecs_ptr
      mwPointer rvecs_ptr

C     Array information:
      mwSize Hrows, Hcols, Brows, Bcols, cols
      mwSize sizeH, sizeB

C     Arguments for computational routine:      
      real*8, allocatable :: H_eff(:,:), B_eff(:,:)
      real*8, allocatable :: ALPHAR(:), ALPHAI(:)
      real*8, allocatable :: BETA(:)
      real*8, allocatable :: l_eigvec(:,:)
      real*8, allocatable :: r_eigvec(:,:)     

      real*8 Work1(1)
      real*8, allocatable :: Work(:)
      
      mwSignedIndex Lwork, INFO


C-----------------------------------------------------------------------
C     Check for proper number of arguments. 
      if(nrhs .ne. 2) then
         call mexErrMsgIdAndTxt ('MATLAB:timestwo:nInput',
     +                           'Two inputs required.')
      elseif(nlhs .gt. 5) then
         call mexErrMsgIdAndTxt ('MATLAB:timestwo:nOutput',
     +                           'Too many output arguments.')
      endif

C     Validate inputs
      
C     Check that the input is a number.
      if ((mxIsNumeric(prhs(1)) .eq. 0)  .or.           
     +        (mxIsNumeric(prhs(2)) .ne. 1)) then
         call mexErrMsgIdAndTxt ('MATLAB:timestwo:NonNumeric',
     +                           'Input must be a number.')
      endif
      
C     Check if inputs are complex.
      if ((mxIsComplex(prhs(1)) .ne. 0) .or.           
     +        (mxIsComplex(prhs(2)) .ne. 0)) then
         call mexErrMsgIdAndTxt ('MATLAB:convec:NonComplex',
     +                           'Inputs must be real.')
      endif    

C     Get the size of the input array. (MATLAB)
      Hrows = mxGetM(prhs(1))
      Hcols = mxGetN(prhs(1))
      Brows = mxGetM(prhs(2))
      Bcols = mxGetN(prhs(2))
      sizeH = Hrows*Hcols
      sizeB = Brows*Bcols
      
      
      if ((Hrows .ne. Brows) .or.           
     +        (Hcols .ne. Bcols)) then
         call mexErrMsgIdAndTxt ('MATLAB:zggev:WrongSize',
     +       'The dimensions of the matrices must equal.')
      endif            
                    
     
C     Allocating memory for the matrices
      allocate(H_eff(Hrows,Hcols))
      allocate(B_eff(Brows,Bcols))
      allocate(ALPHAR(Brows))
      allocate(ALPHAI(Brows))
      allocate(BETA(Brows))
      allocate(l_eigvec(Brows,Bcols))
      allocate(r_eigvec(Brows,Bcols))
C
C     Check the allocated arrays
      if (.not. ( allocated(H_eff) .and.
     + allocated(H_eff) .and.
     + allocated(B_eff) .and.
     + allocated(ALPHAR) .and.
     + allocated(ALPHAI) .and.
     + allocated(BETA) .and.
     + allocated(l_eigvec) .and.
     + allocated(r_eigvec) )) then
         call mexErrMsgIdAndTxt ('MATLAB:zggev:BadAllocation',
     +       'The allocation of variables was unsuccesfull.')
      endif
C
C     Copy input data to the Fortran arrays
      call mxCopyPtrToReal8(mxGetPr(prhs(1)),
     +                          H_eff,sizeH)
     
      call mxCopyPtrToReal8(mxGetPr(prhs(2)),
     +                          B_eff,sizeB)     

      Lwork = -1
C     Query for the workspace
      call DGGEV( 'V', 'V', Hcols, H_eff, Hrows,
     + B_eff, Brows, 
     + ALPHAR, ALPHAI, BETA, l_eigvec, Hrows, r_eigvec, Hrows, 
     + Work1, Lwork, INFO )
C   
C     Test the resulted INFO flag
      call CheckINFO( INFO )
C     
      Lwork = Work1(1)
      allocate( Work(Lwork))
      if (.not. ( allocated(Work) )) then
         call mexErrMsgIdAndTxt ('MATLAB:zggev:BadAllocation',
     +       'The allocation of variable Work was unsuccesfull.')
      endif
C
C     Call the computational subroutine.     
      call DGGEV( 'V', 'V', Hcols, H_eff, Hrows,
     + B_eff, Hrows, 
     + ALPHAR, ALPHAI, BETA, l_eigvec, Hrows, r_eigvec, Hrows, 
     + Work, Lwork, INFO )
C   
C     Test the resulted INFO flag
      call CheckINFO( INFO )
     
C     Create matrices for the return arguments.
      cols = 1
      plhs(1) = mxCreateDoubleMatrix(Hrows,cols,0) ! ALPHAR
      plhs(2) = mxCreateDoubleMatrix(Hrows,cols,0) ! ALPHAI
      plhs(3) = mxCreateDoubleMatrix(Hrows,cols,0) ! BETA
      plhs(4) = mxCreateDoubleMatrix(Hrows,Hcols,1) ! l_eigvec
      plhs(5) = mxCreateDoubleMatrix(Hrows,Hcols,1) ! r_eigvec
      
C     Load the data into to MATLAB outputs.      
      ALPHAR_ptr = mxGetPr(plhs(1))
      call mxCopyReal8ToPtr(ALPHAR,ALPHAR_ptr, Hrows)
C
      ALPHAI_ptr = mxGetPr(plhs(2))
      call mxCopyReal8ToPtr(ALPHAI,ALPHAI_ptr, Hrows) 
C      
      BETA_ptr = mxGetPr(plhs(3)) 
      call mxCopyReal8ToPtr(BETA,BETA_ptr, Hrows)  
C      
      lvecs_ptr = mxGetPr(plhs(4))     
      call mxCopyReal8ToPtr(l_eigvec, lvecs_ptr,sizeB) 
C      
      rvecs_ptr = mxGetPr(plhs(5))
      call mxCopyReal8ToPtr(r_eigvec, rvecs_ptr,sizeB)
  
     

C     Free the allocated matrices
      deallocate(H_eff)
      deallocate(B_eff)
      deallocate(ALPHAR)
      deallocate(ALPHAI)
      deallocate(BETA)
      deallocate(l_eigvec)
      deallocate(r_eigvec)
      deallocate(Work)


      return
      end



C=========================================================
C      Test the output INFO flag of the LAPACK function
      subroutine CheckINFO( INFO )

      mwSignedIndex INFO
      character(len=100) :: msg
      
      if (INFO<0) then
        msg = 'Illegal argumemnt in the LAPACK function'
        call mexErrMsgIdAndTxt ('MATLAB:zggev:IllegalLAPACKArgument',
     +       msg)
      elseif (INFO>0) then
        msg = 'Calculations failed in the LAPACK function'
        call mexErrMsgIdAndTxt ('MATLAB:zggev:FailedLAPACK',
     +       msg)
      endif

      return
      end subroutine


