#include "fintrf.h"
C======================================================================
C     Gateway routine to call the zggev.f function from LAPACK package.
C     (See: http://www.netlib.org/lapack/explore-html/d9/d52/dggev_8f.html for details)
C     Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
C
C     This program is free software: you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation, either version 3 of the License, or
C     (at your option) any later version.
C 
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program.  If not, see http://www.gnu.org/licenses/.
C     
C======================================================================
C     Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C     Declarations
      implicit none

C     mexFunction arguments:
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs


C     Function declarations:
      mwPointer mxGetJc
      mwPointer mxGetIr
      mwPointer mxCreateString
      mwPointer mxGetString, getstring_stat 
      integer mxIsChar

C     Arguments for copying data between fortran and MATLAB
      integer*4 complexflag

      mwPointer mxGetPr
      mwPointer mxCreateDoubleMatrix
      mwPointer mxGetM, mxGetN

      logical fileExist
      character(len=100) :: filename 
      character(len=7) :: file_status

C     Array information:
      mwPointer mxRows, mxCols !size of the input string containing the filename
      integer*4 strlen ! the length of the input string

C     Arguments for computational routine:  
C     fortran representation of the input matrix (CSR format)
      integer*4 rows, cols
      real*8, allocatable ::  mtx(:,:) 
      mwPointer mxMtx
        
C     Parameters for pardiso
      integer i,j

#ifdef DEBUG
C     Debug parameters
      integer*4 mexPrintf
      integer*4 tmp
      character*800 line
#endif


C-----------------------------------------------------------------------
C     Check for proper number of arguments. 
      if(nrhs .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:import_real_full:nInput',
     +                           'One input required.')
      elseif(nlhs .gt. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:import_real_full:nOutput',
     +                           'Too many output arguments.')
      endif

C     Validate inputs      

C     The input filename must be a string.
       if(mxIsChar(prhs(1)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:import_real_full:NonString',
     +                 'Input filename must be a string.')
       endif

C     Get the size of the input string.
       mxRows = mxGetM(prhs(1))
       mxCols = mxGetN(prhs(1))

C     Get the length of the input string and validate.
       strlen = int(mxRows*mxCols, 4)
       if (strlen .gt. len(filename)) then
         call mexErrMsgIdAndTxt ('MATLAB:import_real_full:NonDouble',
     +             'input filename is greater than max str size.')   
       endif

C     Get the input filename.
      getstring_stat = 
     + mxGetString(prhs(1), filename, int(len(filename), 8))

#ifdef DEBUG
      write(line,*) 'Input file name: ',filename, NEW_LINE('A')
      tmp = mexPrintf(line)
#endif



C     Read in the matrix from the binary
C     Testing the existance of the input file containing the sparse matrix A    
      inquire(file=trim(filename),exist=fileExist)!check if it exist
      if(.not.fileExist)then
         call mexErrMsgIdAndTxt ('MATLAB:import_real_full:FileNotFound',
     +             'input file ', trim(filename), 'not found.')  
      endif

C     Opening the file to read in data   
      OPEN(UNIT=6, FILE=trim(filename),STATUS='OLD'
     + ,FORM='UNFORMATTED')

      READ(6,END=999,ERR=1000) rows
      READ(6,END=999,ERR=1000) cols

#ifdef DEBUG
      write(line,*) 'rows: ',rows, ' cols: ', cols, NEW_LINE('A')
      tmp = mexPrintf(line)
#endif

      allocate(mtx(rows,cols))
      if ( .not.(allocated(mtx) )) then
         call mexErrMsgIdAndTxt (
     +                  'MATLAB:import_real_full:BadAllocation',
     +             'Memory space failed to allocate for the matrix.')   
      end if

      do j = 1, cols
        READ(6,END=999,ERR=1000) mtx(:,j)   
      end do


C     Closing the input file
      CLOSE(UNIT=6)


      complexflag = 0;
      mxRows = int( rows, 8)
      mxCols = int( cols, 8)
      plhs(1) = mxCreateDoubleMatrix(mxRows, mxCols, 
     + complexflag) 
      mxMtx = mxGetPr(plhs(1))
      call mxCopyReal8ToPtr(mtx, mxMtx,mxRows*mxcols) 

      deallocate(mtx)
      return

 999  call mexErrMsgIdAndTxt ('MATLAB:import_real_full:FileEnd',
     +   'File end reached while reading the file')        

 1000 call mexErrMsgIdAndTxt ('MATLAB:import_real_full:FileError',
     +   'Error occured while reading the file')        



      return
      

      end


