!#include "fintrf.h"
!======================================================================
!     Gateway routine to call the zggev.f function from LAPACK package.
!     (See: http://www.netlib.org/lapack/explore-html/d9/d52/dggev_8f.html for details)
!     Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
!
!     This program is free software: you can redistribute it and/or modify
!     it under the terms of the GNU General Publi! License as published by
!     the Free Software Foundation, either version 3 of the License, or
!     (at your option) any later version.
! 
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program.  If not, see http://www.gnu.org/licenses/.
!     
!======================================================================
!     Gateway routine
      PROGRAM MatrixIO

      use getPartialInv


!     Declarations
      implicit none


      integer::narg,cptArg !#of arg & counter of arg
      character(len=100):: name !Arg name
      logical LookForFilename, LookForinvDim
      logical LookForOutput
      logical fileExist
      character(len=100) :: filename, output

!     Array information:
      integer*4 nonzerosA !number of nonzeros elements in A
      integer*4 sizeAm, sizeAn !size of the matrix A (rows, and cols) 
      integer*4 sizeInv !the size of the Schur complement to be calculated
      character(len=20) sizeInv_string !The size of the Schur complement -- string from the input
      character(len=8) out_status

!     Arguments for computational routine:  
      integer*4, allocatable :: ia(:), ja(:)
#ifdef COMP
      COMPLEX*16, ALLOCATABLE ::  a(:)    
#else
      REAL*8, ALLOCATABLE ::  a(:)    
#endif       

#ifdef COMP
      COMPLEX*16, allocatable ::  invA(:,:)  
#else
      real*8, allocatable ::  invA(:,:)  
#endif
!      
        
      integer i,j

#ifdef DEBUG
!     Debug parameters
      integer*4 tmp
      character*800 line
#endif

!     https://genomeek.wordpress.com/2012/02/09/fortran-command-line-arguments/
      !Check if any arguments are found
      LookForFilename = .FALSE.
      LookForinvDim = .FALSE.
      LookForOutput = .FALSE.
      filename = ''
      output = ''
      sizeInv = 0
      
      narg=command_argument_count()
      !Loop over the arguments
      if(narg>0)then
      !loop across options
        do cptArg=1,narg
          call get_command_argument(cptArg,name)
          select case(adjustl(name))
          case("--help","-h")
            write(0,*)"This is program calculates the " , &
         "Schur complement of a sparse matirix. Usage: ./getSchur ",  &
         "--filename FILENAME --invDim INTEGER"
            stop
          case("--filename","-F", "--Filename")
            LookForFilename = .TRUE.
          case("--output","-o", "--Output")
            LookForOutput = .TRUE.
          case("--invDim","-I")
            LookForinvDim = .TRUE.
          case default
            if (LookForFilename) then
              filename=adjustl(name) !assign a value to pedfile
              LookForFilename = .FALSE.
            elseif (LookForOutput) then
              output=adjustl(name) !assign a value to pedfile
              LookForOutput = .FALSE.
            elseif (LookForinvDim) then 
              sizeInv_string = trim(adjustl(name))
              READ( sizeInv_string, * ) sizeInv
              LookForinvDim = .FALSE.
            else
              write(0,*)"Option ",adjustl(name),"unknown"
            end if
          end select
        end do
      end if

      if (output.eq.'') then
        output = filename
      end if

#ifdef DEBUG
      write(0,*) 'Input file name:', trim(filename)
      write(0,*) 'Output file name:', trim(output)
#endif

!     Testing the existance of the input file containing the sparse matrix A    
      inquire(file=trim(filename),exist=fileExist)!check if it exist
      if(.not.fileExist)then
        write(0,*)"file ",trim(filename)," not found!"
        stop
      endif

!     Opening the file to read in data   
      OPEN(UNIT=6, FILE=trim(filename),STATUS='OLD',FORM='UNFORMATTED')

      READ(6,END=999,ERR=1000) sizeAn
      READ(6,END=999,ERR=1000) nonzerosA

      allocate(ia(sizeAn+1))
      allocate(ja(nonzerosA))
      allocate(a(nonzerosA))

      READ(6,END=999,ERR=1000) ia(:)
      READ(6,END=999,ERR=1000) ja(:)
      READ(6,END=999,ERR=1000) a(:)



!     Closing the input file
      CLOSE(UNIT=6)

#ifdef DEBUG
      write(0,*) 'Size of the partial inverse:', sizeInv
      write(0,*) 'The size of the input MATRIX:', sizeAn
      write(0,*) 'Nonzeros in the input matrix:', nonzerosA     
      write(0,*) 'The input MATRIX:'
      i = 0
      do j = 1, size(a)
        if (j >= ia(i+1)) then
          i=i+1
        end if

        if (j < ia(i+1)) then  
          !write(0,*) 'row', i, 'col', ja(j), 'element of A = ', a(j)
        end if
      end do

#endif




!     Calculating the partial inverse
      ALLOCATE(invA(sizeInv, sizeInv))

      CALL dgetPartialInv(sizeAn, nonzerosA, a, ia, ja, sizeInv, invA)

!     deallocate the fortran matrix A
      deallocate(ia)
      deallocate(ja)
      deallocate(a)
#ifdef DEBUG
      do i=1, sizeInv
        do j=1, sizeInv
          write(0,*) 'row', i, 'col', j, ' element of A = ', invA(i,j)
        end do
      end do
#endif

      
!     Deallocating memory for the partial inverse  

      if ( allocated(invA) ) then
        DEALLOCATE(invA)
      endif      


      stop

 999  write(*,'(/"End-of-file when i = ",I5)') i
      stop

 1000 write(*,'(/"ERROR reading when i = ",I5)') i
      stop
      end


