%%    Eotvos Quantum Transport Utilities - Messages
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @addtogroup basic Basic Functionalities
%> @{
%> @file Messages.m
%> @brief A class containing methodes for displaying several standard messages.
%> @} 
%> @brief A class containing methodes for displaying several standard messages.
%%
classdef Messages < handle
    
    properties ( Access = protected )
%> An instance of structure #Opt
        Opt
    end


methods
%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure #Opt.
%> @return An instance of the class Messages
    function obj = Messages( Opt )
        if nargin > 0
            obj.Opt = Opt;
        else
            obj.Opt = structures('Opt');
        end
    end

%% display
%> @brief Displays output messages on the screen
%> @param message String containing the message to be displayed
%> @param nosilent Set true to override the silent option given in #Opt.Silent.
    function display( obj, message, nosilent )
        
        if ~exist('message', 'var')
            obj.display( ['Class ', class(obj), ' is created.']);
            return
        end
        
        if isprop('Silent', obj)
            if ~obj.Silent || exist('nosilent', 'var')
                disp( message )
            end
        else        
            if ~obj.Opt.Silent || exist('nosilent', 'var')
                disp( message )
            end
        end
       
        if obj.Opt.debug
            fid = fopen('debug.txt', 'a');
            fprintf(fid, [message,'\n']);
            fclose(fid);
        end
        
    end
    
    
%% getOpt
%> @brief Retrives the structure containing the calculation parameters
%> @return Return an instance of structure #Opt.
    function ret = getOpt( obj )
        ret = obj.Opt;
    end

end % methods public

methods (Static = true)
%% BadInputType
%> @brief Throws a "bad input type" warning, with using the default value.
%> @param variable A string conatining the name of the variable.
%> @param type A string describing the desired type.
    function BadInputType(variable,type)
           warning('MATLAB:BadArgument',['Argument "',variable,'" is not a(an) ',type,'. Setting it to default.']);
    end

%% BadInputTypeNoDefault
%> @brief Throws a "bad input type" warning without setting it to default.
%> @param variable A string conatining the name of the variable.
%> @param type A string describing the desired type.
    function BadInputTypeNoDefault(variable,type)
           warning('MATLAB:BadArgument',['Argument "',variable,'" is not a(an) ',type,'.']);
    end

%% ExceedIteration
%> @brief Throws an "iteration exceeded" warning.
    function ExceedIteration()
           warning('MATLAB:IterExceed','Number of iteration steps reached its limit. Return with initial values!');
    end


end % methods static

end
