%%    Eotvos Quantum Transport Utilities - Parallel
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @addtogroup basic Basic Functionalities
%> @{
%> @file Parallel.m
%> @brief A class for controlling the parallel pool for paralell computations. (compatble only wih MATLAB)
%> @} 
%> @brief A class for controlling the parallel pool for paralell computations. (compatble only wih MATLAB)
%> @Available
%> EQuUs v4.6 or later
%%
classdef Parallel <handle & Messages & CommonFunctions


properties ( Access = protected )
    %>The parallel.Pool object, see the documentation for more details.
    poolobj
    %>Number of attempts to open the parallel pool.
    try2open
    %>Maximal number of attemts to open the parallel pool.
    try2open_max
    %>A period to wait between the attempts to open the parallel pool.
    waitsec
end

properties ( Access = public )
    %> Number of workers
    NumWorkers
end

methods
%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @return An instance of the class
    function obj = Parallel( Opt )
        obj = obj@Messages( Opt );
        obj.poolobj      = [];
        obj.try2open     = 0;
        obj.try2open_max = 20;
        obj.waitsec      = 30;
    end

%% openPool
%> @brief Opens a parallel pool for parallel computations.
    function openPool( obj )       
        
    if obj.isOctave()
        % Parallell computation of EQuUs is not yet supported in octave
        obj.poolobj = [];
        obj.NumWorkers = 0;
    end
       
    obj.try2open = obj.try2open + 1;
        
    %try to open the parallel pool
    try

    if exist('parpool' )
        poolobj = gcp('nocreate');
        if isempty(poolobj)
            if ~isempty(obj.Opt.workers) && obj.Opt.workers>0
                obj.poolobj = parpool(obj.Opt.workers);
                obj.NumWorkers = obj.poolobj.NumWorkers;
            elseif  (~isempty(obj.Opt.workers) && obj.Opt.workers<=0) || isempty(obj.Opt.workers)
                obj.poolobj = [];
                obj.NumWorkers = 0;
            else
                obj.poolobj = parpool();
                obj.NumWorkers = obj.poolobj.NumWorkers;
            end            
        else
            % if parallel pool is already opened, nothing is done here
            obj.poolobj = [];
            obj.NumWorkers = poolobj.NumWorkers;
        end
    else
            if 0 == matlabpool('size')
                if ~isempty(obj.Opt.workers) && obj.Opt.workers>0
                    matlabpool(obj.Opt.workers);
                    obj.poolobj = struct('NumWorkers', obj.Opt.workers);
                elseif (~isempty(obj.Opt.workers) && obj.Opt.workers<=0)  || isempty(obj.Opt.workers)
                    obj.poolobj = [];
                else
                    matlabpool();
                    obj.poolobj = struct('NumWorkers', matlabpool('size'));
                end
            else
                % if parallel pool is already opened, nothing is done here
                obj.poolobj = [];
            end
            obj.NumWorkers = matlabpool('size');
    end
        
    % try to open the parallel pool again
    catch errCause            
        if obj.try2open >= obj.try2open_max
            err = MException('Parallel:openPool', 'Error occured when opening the parallel pool.');
            err = addCause(err, errCause);
            save('Error_Parallel_openPool.mat');
            throw( err );
        else
            obj.display( ['Failed to open the parallel pool. Try to open again in ', num2str(obj.waitsec), ' seconds.'], 1)
            pause( obj.waitsec );
            obj.openPool()     
        end
            
            
    end
       
    end    
    
%% closePool 
%> @brief Closes a parallel pool. 
    function closePool( obj )
        if ~isempty(obj.poolobj)
            if exist('parpool' )
                delete(obj.poolobj)
            else
                matlabpool('close');
            end
        end   
        
        obj.NumWorkers = [];
    end
    
    
    
    

    end % methods

end
