#ifdef __cplusplus
extern "C" {
#endif

/* element prototype to interface with MATLAB */
typedef struct {
  int* dim;
  int* dims;
  void* value;
  mxClassID* classID;
  char* elementname;
} EQelement;

/* structure prototype to interface with MATLAB */
typedef struct {
  EQelement* fields;
  int numfields;
} EQstruct;


#ifdef __INTEL_COMPILER
void equus_c_mp_deallocateelement_( EQelement* eq_element );
#else
void __equus_c_MOD_deallocateelement( EQelement* eq_element );
#endif  

#ifdef __cplusplus
 }
#endif

/** Converts a C string into fortran compatible character array.
 \param cstring The C string to be converted into fortran compatible character array.
 \param strlength The length of the string.
*/
void convert2FortranString( char* cstring, int strlength );

/** Set a field in a MATLAB structure.
 \param eq_element a pointer to an EQelement element prototype.
 \param Mstruct pointer to the MATLAB structure object.
*/
void setField( EQelement* eq_field, mxArray* Mstruct );

/** Set a field in a MATLAB structure. If the given field already exists, then converts the given field into a cell format.
 \param field a pointer to an mxArray containing the data.
 \param Mstruct pointer to the MATLAB structure object.
*/
void setField( mxArray* field, const char* fieldname, mxArray* Mstruct );

/** Get all the element names stored in the eq_struct structure prototype.
 \param eq_struct a pointer to an EQstruct structure prototype.
      \return Pointer to the fieldnames
*/
char** getFieldnames( EQstruct* eq_struct );

/** Sort out invalid EQelements from an EQstruct structure prototype.
 \param eq_struct a pointer to an EQstruct structure prototype.
*/
void regularizeEQstruct( EQstruct* eq_struct );
